require "defines"

-- Configuration
local HEAT_DAMAGE_RATIO = 0.2
local DEFAULT_HEAT_TOLERANCE = 200
local heat_relevant = {["storage-tank"]=true, ["pump"]=true}
local heat_tolerances = {
	["storage-tank"]        = 300,
	["small-pump"]          = 500,
	["heated-storage-tank"] = 2500,
	["heavy-pump"]          = 2500,
	-- Add known third-party storage tanks here. By default, all third-party tanks are IGNORED and heat processing is never applied to them.
}

-- Persistence
local vulnerable_tanks = {}


local function dbgout(msg)
	if #game.players == 0 then return end
	local player = game.players[1]
	
	msg = tostring(msg)
	
	local top = player.gui.top
	if not top.smelterDebugFrame then
		top.add{type="frame", name="smelterDebugFrame",direction="vertical"}
		if not top.smelterDebugFrame.smelterDebugOut then
			top.smelterDebugFrame.add{type="label", name="smelterDebugOut",caption=msg}
			top.smelterDebugFrame.add{type="button", name="smelterDebugClose",caption="Dismiss"}
		end
	else
		top.smelterDebugFrame.smelterDebugOut.caption = msg
	end
end

local function check_debug_button(event)
	local player = game.players[event.player_index]
	if event.element == player.gui.top.smelterDebugFrame.smelterDebugClose then
		player.gui.top.smelterDebugFrame.destroy()
	end
end


-- Initialise
function on_init()
	if nil == global.smelting_heat_vulnerable_tanks then
		-- Remember a list of heat-vulnerable tanks that molten ore should damage.
		global.smelting_heat_vulnerable_tanks = {}
	end
	
	vulnerable_tanks = global.smelting_heat_vulnerable_tanks;
end

function on_load()
	on_init()

	-- Make sure storage tanks are in fast-replaceable.
	if game and game.entity_prototypes and game.entity_prototypes["storage-tank"] then
		game.entity_prototypes["storage-tank"].fast_replaceable_group = "storage-tank"
	end
end

local function pstr(pos)
	if pos.x then
		return tostring(pos.x) .. "," .. tostring(pos.y)
	else
		return tostring(pox[0]) .. "," .. tostring(pos[1])
	end
end

-- Table-keeping.
function check_built(event)
	if heat_relevant[event.created_entity.type] and heat_tolerances[event.created_entity.name] then
		table.insert(vulnerable_tanks, event.created_entity)
		--dbgout("vulnerable tank at " .. pstr(event.created_entity.position) .. ", total " .. tostring(#vulnerable_tanks))
	end
end

-- Updates
local function heat_process(tank)
	local tolerance = heat_tolerances[tank.prototype.name] or DEFAULT_HEAT_TOLERANCE
	local fluid = tank.fluidbox
	if not fluid then return end
	
	for i = 1, #fluid do
		local box = fluid[i]
		if box and box.amount > 0.1 and box.temperature > tolerance then
			tank.damage((box.temperature - tolerance) / tolerance * HEAT_DAMAGE_RATIO, tank.force, 'fire')
		end
	end
end

function on_tick(event)
	local n = game.tick % 12	-- every 12th tick = 10 times/sec
	if n == 0 and #vulnerable_tanks > 0 then
		for i, tankage in ipairs(vulnerable_tanks) do
			if not tankage.valid then
				table.remove(vulnerable_tanks, i)
			else
				heat_process(tankage)
			end
		end
	end
end


-- Event Registration
script.on_load(on_load)
script.on_init(on_init)

script.on_event(defines.events.on_gui_click, check_debug_button)
script.on_event(defines.events.on_built_entity, check_built)
script.on_event(defines.events.on_robot_built_entity, check_built)
script.on_event(defines.events.on_tick, on_tick)
